/* 
 * output.cc -- ePiX file-formatting output routines
 *
 * Andrew D. Hwang                      <ahwang@mathcs.holycross.edu>
 *
 * Version: 0.7.x
 * Last Change: May 27, 2002
 */

#define _EPIX_COMPILE
#define _VERSION // *version accessible from this file only
#include "output.h"

double h_size, v_size, h_offset, v_offset;
double viewpt1, viewpt2, viewpt3;
double x_min, x_max, y_min, y_max, x_size, y_size;
double pic_size;
char *pic_unit;
extern const char *version;

extern int lines_printed;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *                 Sectioning Functions                  *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* Print standard ePiX picture header */
void begin(void)
{
  fprintf (stdout, "%%%% File generated by ePiX-%s", version);
  fprintf (stdout, "\n%%%%");
  fprintf (stdout, "\n%%%%   Cartesian bounding box: [%g,%g] x [%g,%g]",
	 x_min, x_max, y_min, y_max);
  fprintf (stdout, "\n%%%%   Actual size: %g%s x %g%s",
	 h_size*pic_size, pic_unit, v_size*pic_size, pic_unit);
  fprintf (stdout, "\n%%%%   Figure offset: %s by %g%s, %s by %g%s",
        (h_offset >= 0) ? "right" : "left", fabs(h_offset)*pic_size, pic_unit, 
        (v_offset >= 0) ? "up"    : "down", fabs(v_offset)*pic_size, pic_unit);
  fprintf (stdout, "\n%%%%");
  fprintf (stdout, "\n\\setlength{\\unitlength}{%g%s}", pic_size, pic_unit);
  fprintf (stdout, "\n\\begin{picture}(%g,%g)(%g,%g)", \
	 h_size, v_size, -h_offset, -v_offset);
  end_stanza();
}

/* End picture */
void end(void)
{
  fprintf (stdout, "\n\\end{picture}\n");
}

/* Print Cartesian pair in LaTeX coordinates */
void print(pair p)
{
  pair out = c2p(p);
  fprintf (stdout, "(%g,%g)", out.x1, out.x2);
}
/* Print pair without conversion */
void raw_print(pair p)
{
  fprintf (stdout, "(%.2f,%.2f)", p.x1, p.x2);
}

/* Path style declarations */
void start_path(const int path_style)
{
  fprintf (stdout, "\n");

  if (path_style == PLAIN)
    fprintf (stdout, "\\path");

  else if (path_style == DASHED)
    fprintf (stdout, "\\dashline{%.1f}", DASH_LENGTH);

  else if (path_style == DOTTED)
    fprintf (stdout, "\\dottedline{%.1f}", DOT_SEP);

  else if (path_style == BOLD)
    fprintf (stdout, "\\path");

  else
    fprintf (stdout, "\\path");
}

/* Fill closed path */
void fill(void)
{
  fprintf (stdout, "\\blacken");

}

/* End current stanza of .eepic file */
void end_stanza(void)
{
  int i=0;
  /* Stanza separator of width FILE_WIDTH */
  fprintf (stdout, "\n%%%%\n%%%%");
  for (i=0; i<= FILE_WIDTH - 4; ++i)
    fprintf (stdout, "-");
  fprintf (stdout, "%%%%");

  output();
}

void fatal_error(void)
{
  int i;
  fprintf (stdout, "\n%%%%\n");
  for (i=0; i <= FILE_WIDTH; ++i)
    fprintf (stdout, "%%");

  fprintf (stdout, "\n%%%%      ");
  fprintf (stdout, "That's more than %d lines. I suspect an error...", MAX_LINES);
  fprintf (stdout, "     %%%%\n");

  for (i=0; i <= FILE_WIDTH; ++i)
    fprintf (stdout, "%%");
  end();
  exit(1);
}

void output(void)
{
  if (lines_printed > MAX_LINES)
    fatal_error();
}

void line_break(int i, int num_pts, int path_style)
{
  // Print only 3 points on first line of dashed/dotted path
  if (path_style == DASHED || path_style == DOTTED)
    ++i;
  
  if ((i+1) % PAIRS_PER_LINE == 0 && i < num_pts)
    {
      fprintf (stdout, "\n  ");
      ++lines_printed;
      output();
    }
}

/* Output styles */

void
bold(void)
{
  fprintf (stdout, "\n{\\thicklines");
}
void
endbold(void)
{
  fprintf (stdout, "\n}%%");
}
void
end_bold(void)
{
  fprintf (stdout, "\n}%%");
}

/* Variable color handling, requires "color" package */

/* Force double to [0,1] */
static double 
clip_to_unit(double t)
{
  if (t < 0)
    return 0;
  else if (t > 1)
    return 1;
  else
    return t;
}

/* Red-Blue-Green color model */
void
rgb(double r, double g, double b)
{
  r = clip_to_unit(r);
  g = clip_to_unit(g);
  b = clip_to_unit(b);
  fprintf (stdout, "\n\\color[rgb]{%.2f,%.2f,%.2f}", r, g, b);
}

/* Cyan-Magenta-Yellow-Black color model, better for printing */
void
cmyk(double c, double m, double y, double k)
{
  c = clip_to_unit(c);
  m = clip_to_unit(m);
  y = clip_to_unit(y);
  k = clip_to_unit(k);
  fprintf (stdout, "\n\\color[cmyk]{%.2f,%.2f,%.2f,%.2f}", c, m, y, k);
}

void black(void)
{
  fprintf (stdout, "\n\\color[cmyk]{0,0,0,1}");
}

/* Static Color handling */
void
red(void)
{
  rgb(1,0,0);
}
void
end_red(void)
{
  black();
}

void
blue(void)
{
  rgb(0,0,1);
}
void
end_blue(void)
{
  black();
}

void
green(void)
{
  rgb(0,1,0);
}
void
end_green(void)
{
  black();
}

void
cyan(void)
{
  cmyk(1,0,0,0);
}
void
end_cyan(void)
{
  black();
}

void
magenta(void)
{
  cmyk(0,1,0,0);
}
void
end_magenta(void)
{
  black();
}

void
yellow(void)
{
  cmyk(0,0,1,0);
}
void
end_yellow(void)
{
  black();
}

